#include <ESP8266WiFi.h>
#include <TimeLib.h>
#include <WiFiUdp.h>
#include <Adafruit_NeoPixel.h>

#include "display.h"
#include "wifi.h"
#include "ntp.h"

WiFiClient espClient;
time_t prevDisplay = 0;
int totalMinutes = 0;
int totalSeconds = 0;
int wakeUpTime = 1800;        //30 minutes is 1800 seconds

int organic = false;
int binairy = false;
int wakeUp = false;
int alarm = false;

float floatTime;
float organicStart = 8.5;
float organicEnd = 24.0;
float alarmStart = 0;
float alarmEnd = 8.0;
float wakeUpStart = alarmEnd;
float wakeUpEnd = wakeUpStart + 0.5;

int localHour = 0;
int localMinute = 0;
int teller = 0;

bool debug = false;
bool autoDebug = false;
bool colon;

void setup() {
  Serial.begin(9600);
  delay(500);
  //hello world
	Serial.println("start");
  Serial.println("Connecting to WiFi");
	//wifi and NTP
  wifiSetup();
  Udp.begin(localPort);
	//debug Serial.print("Local port: ");
	//debug Serial.println(Udp.localPort());
	Serial.println("waiting for synchronization");
	while (timeStatus() == timeNotSet){
		setSyncProvider(getNtpTime);
	}
	//synchronisation interval
	setSyncInterval(86400);
  //neopixel
  pixels.begin();
  clearAll();  
  test();

}

//generates leading zero´s when value below 2 digits
void printDigits(int digits)
{
	Serial.print(":");
	if (digits < 10)
			Serial.print('0');
	Serial.print(digits);
  Serial.println();
}

//pace is the heart of the clock
//it ticks with every half second and give various functions the actual time
//and it's window to display
void pace(){
  if (debug){
    if (autoDebug){
      //autodebug is used to run the clock in a speed needed for debugging
      //standard speeed is a minute every half second 
      //duration of one day; 720 seconds
      localMinute ++;
      if (localMinute == 60){
        localHour++;
        localMinute = 0;
        if (localHour == 24){
          localHour = 0;
        }
      }
    }else{
      //no autodebug give us the means to pass a time set by hand
      localHour = 0;
      localMinute = 0;
    }
  }else{
    //no debug, clockfunction in order by time from 0 - 24 hours
    //calculate time in convieniant numbers
    localHour = (hour() - getDST(day(), month(), weekday()));
    if (localHour == -1){
		  localHour = 23;
		}
    localMinute = minute();
  }
  //show time
  Serial.print("Current time: ");
  Serial.print(localHour);
  printDigits(localMinute);
  setModus(localHour, localMinute);
  totalMinutes = localHour * 60 + localMinute;    //total amount minutes
  totalSeconds = localMinute * 60 + second();     //total amount seconds
  floatTime = localHour + localMinute / 60.0;
  if (alarm){
    Alarm(totalMinutes);
  }else{
    if (wakeUp){
      //debug Serial.print("floatTime: ");
      //debug Serial.println (floatTime);
      //debug Serial.print("wakeUpStart: ");
      //debug Serial.println (wakeUpStart);
      //wakeUpTime is initialized at 1800 i.e. 30 * 60 min = 1/2 hour
      WakeUp((floatTime - wakeUpStart)/( wakeUpEnd - wakeUpStart) * 100);
    }else{
      if (organic){
        teller++;
        if (teller < 16){
          clearAll();
          Organic(totalSeconds, false);
        }else{
          if (teller < 24){
            clearAll();
            Organic(totalMinutes, true);
          }else{
            teller = 0;
          }
        }     
      }else{
        teller++;
        if (teller < 16){
          Binairy(totalMinutes, true);
        }else{
          if (teller < 20){
            Binairy(totalSeconds, false);
          }else{
            teller = 0;
          }
        }     
      }
    }
  } 
  if (colon){
    colon = false;
  }else{
    colon = true;
  }
  showColon(colon);
  //debug Serial.print("colon: ");
  //debug Serial.println(colon);
  delay(500);
}

//sets the clock modus according to time of day
void setModus(int fUur,int  fMin){
  //start with no modus
  organic = false;
  binairy = false;
  wakeUp = false;
  alarm = false;
  //time is a float of hours and minutes; i.e. 8:30 == 8.5
  floatTime = fUur + fMin / 60.0;
  //debug Serial.println(floatTime);
  //normal modus during daylight and no other modus forced
  //organic display differs from binairy by the interface
  //wich is more human readable
  if (floatTime > organicStart && floatTime < organicEnd){
    organic = true;
  }else{
    //the wake up period in wich the clock will light up from zero to full brightness
    if (floatTime >= wakeUpStart && floatTime <= wakeUpEnd){
      wakeUp = true;
    }else{
      //The period in wich the clock shows the remaining sleeping time
      if (floatTime >= alarmStart && floatTime < alarmEnd){
        alarm = true;
      }else{
        //binairy display
        binairy = true;
      }
    }
  }
  //debug if (organic){
  //debug  Serial.println("organic");
  //debug}
  //debugif (alarm){
  //debug  Serial.println("alarm");
  //debug}
  //debugif (wakeUp){
  //debug Serial.println("wake up");
  //debug}
  //debugif (binairy){
  //debug  Serial.println("binairy");
  //debug}
  //force test modus at any time
  organic = true;
  binairy = false;
  wakeUp = false;
  alarm = false;
}

void loop() {
if (timeStatus() != timeNotSet) {					    //if status time is not set	
		if (now() != prevDisplay) { 							//if now is not equal to prev display value
    pace();
    }
  }
}

//get day light savings time
int getDST(int day, int month, int dow){
	if (month < 3 || month > 10)  return 1; 
	if (month > 3 && month < 10)  return 0; 
	int previousSunday = day - dow;
	if ((month == 3) && (previousSunday >= 25))
	{
		 return 0;
	 } else {
		 return 1;
	 }
	 if ((month == 10) && (previousSunday <25))
	{
		 return 0;
	 } else {
		 return 1;
	 }
	return false; // this line never gonna happend
}


